#version 400

uniform sampler2D s_HeightMap;

uniform float u_SampleDistance;
uniform float u_InverseSampleDistance;
uniform float u_HeightScale;

uniform vec3 u_FaceTransform1;
uniform vec3 u_FaceTransform2;
uniform vec3 u_FaceTransform3;

-- vs
layout(location = 0) in vec4 vs_Position;
layout(location = 1) in vec2 vs_TexCoord1;
layout(location = 2) in vec2 vs_TexCoord2;

out vec2 fs_TexCoord1;
out vec2 fs_TexCoord2;

void main() {
	gl_Position = vs_Position;
	fs_TexCoord1 = vs_TexCoord1;
	fs_TexCoord2 = vs_TexCoord2;
}

-- fs
in vec2 fs_TexCoord1;
in vec2 fs_TexCoord2;

void main() {
	vec2 uv1 = fs_TexCoord1;
	vec2 st1 = fs_TexCoord2;
	
	mat3 faceTransform = mat3(u_FaceTransform1, u_FaceTransform2, u_FaceTransform3);
	
	vec2 xOffset = u_SampleDistance * vec2(1, 0);
	vec2 yOffset = u_SampleDistance * vec2(0, 1);
	
	float xDifference = texture2D(s_HeightMap, uv1 + xOffset).r
					  - texture2D(s_HeightMap, uv1 - xOffset).r;
	float yDifference = texture2D(s_HeightMap, uv1 + yOffset).r
					  - texture2D(s_HeightMap, uv1 - yOffset).r;
					  
	float s = st1.x;
	float t = st1.y;
	float iw = inversesqrt(s * s + t * t + 1.0);
	float h = 1.0 + u_HeightScale * texture2D(s_HeightMap, uv1).r;
	
	float st = s * t;
	float iw2 = iw * iw;
	float iw3 = iw2 * iw;
	float hiw = h * iw;
	float hiw3 = h * iw3;
	
	mat3 jacobian = mat3(
		hiw * (1.0 - s * s * iw2), -st * hiw3, -s * hiw3,
		-st * hiw3, hiw * (1.0 - t * t * iw2), -t * hiw3,
		u_HeightScale * s * iw, u_HeightScale * t * iw, u_HeightScale * iw
	);
	
	vec3 normal = normalize(
		faceTransform * cross(
			jacobian * vec3(2, 0,  xDifference * u_InverseSampleDistance),
			jacobian * vec3(0, 2, -yDifference * u_InverseSampleDistance)
		)
	);
	
	gl_FragColor = vec4(normal * .5+.5, 1);
}